<?xml version = "1.0" encoding = "UTF-8"?>
<!--
_________________________________________________________________________________________________________________________________________________________________________________________
                     
                  Product            : Diagnostic Imaging Report
                  Release            : CDA_Implementation_Guide_v1.0
                  IG Guide Reference : 7.1.1.1
                  IG Guide Title     : IMAGING EXAMINATION RESULT                                                                                                                                                                                         
_________________________________________________________________________________________________________________________________________________________________________________________
-->

<schema xmlns="http://www.ascc.net/xml/schematron">
    <ns prefix="cda" uri="urn:hl7-org:v3"/>
    <ns prefix="ext" uri="http://ns.electronichealth.net.au/Ci/Cda/Extensions/3.0"/>
    <ns prefix="xs" uri="http://www.w3.org/2001/XMLSchema"/>
    <ns prefix="xsi" uri="http://www.w3.org/2001/XMLSchema-instance"/>

    <pattern name="p-Diagnostic_Imaging_Report_3A_IMAGINGEXAMINATIONRESULT_7_1_1_1_custom-errors"
        id="p-Diagnostic_Imaging_Report_3A_IMAGINGEXAMINATIONRESULT_7_1_1_1_custom-errors"
        see="#p-Diagnostic_Imaging_Report_3A_IMAGINGEXAMINATIONRESULT_7_1_1_1_custom-errors">



        <rule
            context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation">

            <assert test="@classCode">Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING EXAMINATION
                RESULT - "IMAGING EXAMINATION RESULT / Examination Result Name (Imaging Examination
                Result Name)" - The 'observation' tag 'classCode' attribute is missing. Refer to
                section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>

            <assert test="not(@classCode) or normalize-space(@classCode) != ''">Error: Diagnostic
                Imaging Report - 7.1.1.1 IMAGING EXAMINATION RESULT - "IMAGING EXAMINATION RESULT /
                Examination Result Name (Imaging Examination Result Name)" - The 'observation' tag
                'classCode' attribute shall contain a value. Refer to section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>

            <assert
                test="
                not(@classCode) or normalize-space(@classCode) = '' or @classCode = 'OBS'"
                >Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING EXAMINATION RESULT - "IMAGING
                EXAMINATION RESULT / Examination Result Name (Imaging Examination Result Name)" -
                The 'observation' tag 'classCode' attribute shall contain the value 'OBS'. Refer to
                section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>


            <report
                test="cda:entryRelationship[cda:observation/cda:code/@code='103.17009'] and not(cda:targetSiteCode)"
                >Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING EXAMINATION RESULT - "IMAGING
                EXAMINATION RESULT / Anatomical Site (ANATOMICAL LOCATION) / SPECIFIC LOCATION /
                Name of Location (Anatomical Location Name)" - The 'Name of Location (Anatomical
                Location Name)' shall be present if 'Anatomical Region' exists. Refer to section
                7.1.1.1 of the Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0. </report>

        </rule>




        <!-- In reference to the test below for "IMAGING EXAMINATION RESULT / Observation DateTime" the IG states the following: 
             "This element will hold the same value as EXAMINATION REQUEST DETAILS / IMAGE DETAILS / Image DateTime (entryRelationship[exam_req]/act/entryRelationship[img_det]/observation/effectiveTime)" 
             But as per the issue raised and then the clarification received from the CI team through Jira reference RPSA-2015 we are considering 'will' as SHALL -->
        <rule
            context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation[@classCode = 'OBS']/cda:entryRelationship/cda:observation[cda:code/@code = '103.16589']/cda:effectiveTime">

            <assert
                test="(not(@value) 
                and 
                not(/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation[@classCode = 'OBS']/cda:entryRelationship/cda:act[cda:code/@code = '102.16511']/cda:entryRelationship/cda:observation[cda:code/@code = '102.16515']/cda:effectiveTime/@value)) 
                or 
                (@value = /cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation[@classCode = 'OBS']/cda:entryRelationship/cda:act[cda:code/@code = '102.16511']/cda:entryRelationship/cda:observation[cda:code/@code = '102.16515']/cda:effectiveTime/@value)"
                >Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING EXAMINATION RESULT - "IMAGING
                EXAMINATION RESULT / Observation DateTime" - The 'effectiveTime' tag 'value'
                attribute value shall contain the same value as 'EXAMINATION REQUEST DETAILS / IMAGE
                DETAILS / Image DateTime
                (entryRelationship[exam_req]/act/entryRelationship[img_det]/observation/effectiveTime)'.
                Refer to section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>

        </rule>


        <rule
            context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation[@classCode = 'OBS']/cda:targetSiteCode">
            
            <report test="@nullFlavor and (@code or cda:originalText or cda:qualifier)">Error: Diagnostic Imaging Report - 7.1.1.1
                IMAGING EXAMINATION RESULT - "IMAGING EXAMINATION RESULT / Anatomical Site
                (ANATOMICAL LOCATION) / SPECIFIC LOCATION / Name of Location (Anatomical Location
                Name)" - For the presence of the 'targetSiteCode' tag 'nullFlavor' attribute,
                no other attributes or elements are permitted. Refer to section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0. </report>
            
            <report test="@nullFlavor and normalize-space(@nullFlavor) != '' and @nullFlavor != 'NI' and @nullFlavor != 'UNK'"
                >Error: Diagnostic Imaging Report - 7.1.1.1
                IMAGING EXAMINATION RESULT - "IMAGING EXAMINATION RESULT / Anatomical Site
                (ANATOMICAL LOCATION) / SPECIFIC LOCATION / Name of Location (Anatomical Location
                Name)" -  The 'targetSiteCode' tag 'nullFlavor' attribute shall contain the value 'NI' or 'UNK'.
                Refer to section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</report>
            
            <report test="not(@code) and cda:qualifier"
                >Error: Diagnostic Imaging Report - 7.1.1.1
                IMAGING EXAMINATION RESULT - "IMAGING EXAMINATION RESULT / Anatomical Site
                (ANATOMICAL LOCATION) / SPECIFIC LOCATION / Name of Location (Anatomical Location
                Name)" -
                For the presence of the 'qualifier' tag the corresponding 'targetSiteCode' tag 'code' attribute shall be present.
                Refer to section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</report>
            
            
            
        </rule>
      
      
        
        <rule  context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation[@classCode = 'OBS']/cda:targetSiteCode[@codeSystem = '2.16.840.1.113883.6.96']/cda:qualifier/cda:value">
            
            <assert test="not(@nullFlavor)">Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING
                EXAMINATION RESULT - "IMAGING EXAMINATION RESULT / Anatomical Site(ANATOMICAL
                LOCATION) / SPECIFIC LOCATION / Side" - The 'value' tag 'nullFlavor' attribute shall
                not be present. Refer to "025157 - Identify the laterality using a SNOMED CT-AU
                code" of the
                eHealth_Diagnostic_Imaging_Report_My_Health_Record_Conformance_Profile_v1.1 and
                section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>
            
            <assert
                test="
                not(@codeSystem) or normalize-space(@codeSystem) = '' or @codeSystem = '2.16.840.1.113883.6.96'"
                >Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING EXAMINATION RESULT - "IMAGING
                EXAMINATION RESULT / Anatomical Site(ANATOMICAL LOCATION) / SPECIFIC LOCATION /
                Side" - The 'value' tag 'codeSystem' attribute shall contain the value
                '2.16.840.1.113883.6.96'. Refer to "025157 - Identify the laterality using a SNOMED
                CT-AU code" of the
                eHealth_Diagnostic_Imaging_Report_My_Health_Record_Conformance_Profile_v1.1 and
                section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert> 
            
            
        </rule>
        
        
        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation[@classCode = 'OBS']/cda:entryRelationship/cda:observation[cda:code/@code = '103.17009']/cda:value">
            
            <assert
                test="
                not(@xsi:type) or normalize-space(@xsi:type) = '' or 
                normalize-space(@xsi:type) = 'CD' or
                (substring-before(normalize-space(@xsi:type), ':') != '' and
                substring-after(normalize-space(@xsi:type), ':') = 'CD') or 
                normalize-space(@xsi:type) = 'CE' or
                (substring-before(normalize-space(@xsi:type), ':') != '' and
                substring-after(normalize-space(@xsi:type), ':') = 'CE')"
                > Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING EXAMINATION RESULT -
                "IMAGING EXAMINATION RESULT / ANATOMICAL SITE DETAILS / Anatomical Region" - The 'value' tag 'xsi:type' attribute shall contain
                the value 'CD' or 'CE'. Refer to section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>
            
            <assert
                test="(@code and @codeSystem) or (cda:originalText and @nullFlavor = 'OTH' and @codeSystem)"
                >Error: Diagnostic Imaging Report - 7.1.1.1 IMAGING EXAMINATION RESULT -
                "IMAGING EXAMINATION RESULT / ANATOMICAL SITE DETAILS / Anatomical Region" - The 'value' tag shall have 'code' and 'codeSystem'
                attribute  or 'originalText' tag, 'codeSystem' attribute and 'nullFlavor' attribute with a value 'OTH'. 
                Refer to section 2.6.2 of the HL7_Version_3_Standard_Data_Types_Abstract_Specification_Release_1 and section
                7.1.1.2 of the Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>
            
        </rule>
        
        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '101.16945']/cda:component/cda:section[cda:code/@code = '102.16145']/cda:entry/cda:observation[@classCode = 'OBS']/cda:entryRelationship/cda:observation[cda:code/@code = '308552006']/cda:value">
            


            <assert
                test="
                not(@xsi:type) or normalize-space(@xsi:type) = '' or 
                normalize-space(@xsi:type) = 'CD' or
                (substring-before(normalize-space(@xsi:type), ':') != '' and
                substring-after(normalize-space(@xsi:type), ':') = 'CD') or 
                normalize-space(@xsi:type) = 'CE' or
                (substring-before(normalize-space(@xsi:type), ':') != '' and
                substring-after(normalize-space(@xsi:type), ':') = 'CE')"
                > Error: Diagnostic Imaging Report - 7.1.1.1
                IMAGING EXAMINATION RESULT - "IMAGING EXAMINATION RESULT / Overall Result Status (Imaging Examination Result Status)" - The 'value' tag 'xsi:type' attribute shall contain
                the value 'CD' or 'CE'. Refer to section 7.1.1.1 of the
                Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>

            <assert
                test="(@code and @codeSystem) or (cda:originalText and @nullFlavor = 'OTH' and @codeSystem)"
                >Error: Diagnostic Imaging Report - 7.1.1.1
                IMAGING EXAMINATION RESULT - "IMAGING EXAMINATION RESULT / Overall Result Status (Imaging Examination Result Status)" - The 'value' tag shall have 'code' and 'codeSystem'
                attribute  or 'originalText' tag, 'codeSystem' attribute and 'nullFlavor' attribute with a value 'OTH'. 
                Refer to section 2.6.2 of the HL7_Version_3_Standard_Data_Types_Abstract_Specification_Release_1 and section
                7.1.1.2 of the Diagnostic_Imaging_Report_CDA_Implementation_Guide_v1.0.</assert>
            
        </rule>


    </pattern>

</schema>
