<?xml version = "1.0" encoding = "UTF-8"?>

<schema xmlns="http://www.ascc.net/xml/schematron">
    <ns prefix = "cda" uri = "urn:hl7-org:v3"/>
    <ns prefix = "ext" uri = "http://ns.electronichealth.net.au/Ci/Cda/Extensions/3.0"/>
    <ns prefix = "xs" uri = "http://www.w3.org/2001/XMLSchema"/>
    <ns prefix = "xsi" uri = "http://www.w3.org/2001/XMLSchema-instance"/>

    <pattern name="p-PCEHR_Dispense_Record_3A_DispenseItem_MEDICATIONACTION_7_1_1_custom-errors"
        id="p-PCEHR_Dispense_Record_3A_DispenseItem_MEDICATIONACTION_7_1_1_custom-errors"
        see="#p-PCEHR_Dispense_Record_3A_DispenseItem_MEDICATIONACTION_7_1_1_custom-errors">



      <!-- 24/10/2012 : RS - This test is added because of the duplication of rule context for substanceAdministration and hence also to
          test the existence of classcode which is part of the rule context in the original file -->

       <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '102.16210']">

           <assert test="normalize-space(cda:entry/cda:substanceAdministration/@classCode) != ''"
               >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
               "Dispense Item" - The 'entry/substanceAdministration' tag 'classCode' attribute shall contain a value.
               Refer to section 7.1.1 of the
               PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.

           </assert>

            <!-- 11/12/2012 : SY - Below PredicatedNodeExistenceTest has been generated inside PCEHR_DispenseItem_MEDICATIONACTION_7_1_1.sch -->
            <!--
           <assert test="cda:entry[cda:substanceAdministration/@classCode='SBADM']"
               >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
               "Dispense Item" - The 'entry' tag is missing for 'Dispense Item Substance Administration'.
               Refer to section 7.1.1 of the
               PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.

           </assert>
            -->

           </rule>


        <!-- 24/10/2012 : RS - This test for code is added because it is suppressed and cannot be generated by Schematron Generator -->

        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '102.16210']/
                        cda:entry/cda:substanceAdministration[@classCode='SBADM']/
                        cda:statusCode">

            <assert test="

                not(@code) or normalize-space(@code) = '' or @code = 'active' or @code = 'completed'"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item" -
                The 'statusCode' tag 'code' attribute shall contain the
                value 'active' or 'completed'. Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</assert>

            <!-- originalText required if no @code present -->
            <report test="
                not(@code) and
                not(cda:originalText)"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item" -
                The 'originalText' tag is missing for without code attribute present.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</report>

            <!-- Don't use OTH at all -->
            <report test="
                @nullFlavor and
                @nullFlavor = 'OTH'"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item" -
                The 'code' tag 'nullFlavor' attribute shall not contain the value 'OTH'.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</report>

        </rule>



        <!-- 17/10/2012 : RS - This test is added because of the duplication of rule context for supply and hence
        test the existence of classcode which was part of the rule context in the original file -->

        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '102.16210']/
                        cda:entry/cda:substanceAdministration[@classCode='SBADM']/
                        cda:entryRelationship/cda:supply">

            <!-- 01/11/2012 - As per the new IG :: NEHTA_931_2012_PCEHR_Dispense_Record_CDA_v1_0_20121026.pdf
                the existence of @classcode need not be tested -->

            <!--<assert test="@classCode"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item" -
                The 'supply' tag 'classCode' attribute is missing.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</assert>
            -->

            <assert test="not(@classCode) or normalize-space(@classCode) != ''"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item" -
                The 'supply' tag 'classCode' attribute shall contain a value.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</assert>


            <assert test="
                not(@classCode) or normalize-space(@classCode) = '' or @classCode = 'SPLY'"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item" -
                The 'supply' tag 'classCode' attribute shall contain the value 'SPLY'.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</assert>

        </rule>


        <!-- 24/10/2012 : RS - This test is added to test the existence of supply which was part of the rule context in the original file -->

        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '102.16210']/
                        cda:entry/cda:substanceAdministration[@classCode='SBADM']">


            <!-- 01/11/2012 - As per the new IG :: NEHTA_931_2012_PCEHR_Dispense_Record_CDA_v1_0_20121026.pdf
            the existence of @classcode need not be tested hence it is removed as a predicate of cda:supply -->

            <assert test="cda:entryRelationship[cda:supply]">Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item" - The 'supply' tag is missing. Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</assert>

        </rule>


        <!-- 18/10/2012 : RS - This test is added because code tag existence test suppressed within the generator database being part of the rule context -->
        <!-- 12/12/2012 : SY - Below NodeExistenceTest has been generated inside PCEHR_DispenseItem_MEDICATIONACTION_7_1_1.sch -->
        <!--
        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '102.16210']/
                        cda:entry/cda:substanceAdministration[@classCode='SBADM']/
                        cda:entryRelationship/cda:supply[@classCode='SPLY']/
                        cda:product/cda:manufacturedProduct/cda:manufacturedMaterial">

            <assert test="cda:code"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item / Therapeutic Good Identification" -
                The 'code' tag is missing. Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</assert>

        </rule>
        -->


        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '102.16210']/
                        cda:entry/cda:substanceAdministration[@classCode='SBADM']/
                        cda:entryRelationship/cda:supply[@classCode='SPLY']/
                        cda:product/cda:manufacturedProduct/cda:manufacturedMaterial/cda:code">

            <!-- originalText required if no @code present -->
            <report test="
                not(@code) and
                not(cda:originalText)"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item / Therapeutic Good Identification" -
                The 'originalText' tag is missing for without code attribute present.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</report>

            <!-- Don't use OTH at all -->
            <report test="
                @nullFlavor and
                @nullFlavor = 'OTH'"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item / Therapeutic Good Identification" -
                The 'code' tag 'nullFlavor' attribute shall not contain the value 'OTH'.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</report>

        </rule>


        <rule context="/cda:ClinicalDocument/cda:component/cda:structuredBody/cda:component/cda:section[cda:code/@code = '102.16210']/
                        cda:entry/cda:substanceAdministration[@classCode='SBADM']/
                        cda:entryRelationship/cda:supply[@classCode='SPLY']/
                        cda:product/cda:manufacturedProduct/cda:manufacturedMaterial/ext:formCode">

            <!-- originalText required if no @code present -->
            <report test="
                not(@code) and
                not(cda:originalText)"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item / Formula" -
                The 'originalText' tag is missing for without code attribute present.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</report>

            <!-- Don't use OTH at all -->
            <report test="
                @nullFlavor and
                @nullFlavor = 'OTH'"
                >Error: PCEHR Dispense Record - 7.1.1 Dispense Item (MEDICATION ACTION) -
                "Dispense Item / Formula" -
                The 'code' tag 'nullFlavor' attribute shall not contain the value 'OTH'.
                Refer to section 7.1.1 of the
                PCEHR_Dispense_Record_CDA_Implementation_Guide_v1.0.</report>

        </rule>



    </pattern>

</schema>
